# Generated by Django 4.0.7 on 2022-11-06 16:52

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveonline", "0016_character_names_are_not_unique"),
        ("eveuniverse", "0007_evetype_description"),
        ("miningtaxes", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="AdminCharacter",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                (
                    "eve_character",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="miningtaxes_admin_character",
                        to="eveonline.evecharacter",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AdminMiningObservers",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("obs_id", models.BigIntegerField()),
                ("obs_type", models.CharField(max_length=32)),
                ("name", models.CharField(max_length=32)),
                ("sys_name", models.CharField(max_length=32)),
                (
                    "character",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="mining_obs",
                        to="miningtaxes.admincharacter",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Character",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                (
                    "eve_character",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="miningtaxes_character",
                        to="eveonline.evecharacter",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Settings",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "phrase",
                    models.CharField(
                        blank=True,
                        default="",
                        max_length=10,
                        verbose_name="Keyword (case sensitive) that must be present in the donation reason to be counted. Leave blank/empty to count all donations regardless of reason.",
                    ),
                ),
                (
                    "tax_R64",
                    models.FloatField(
                        default=10.0, verbose_name="Tax for Moon R64 Ore as a percent"
                    ),
                ),
                (
                    "tax_R32",
                    models.FloatField(
                        default=10.0, verbose_name="Tax for Moon R32 Ore as a percent"
                    ),
                ),
                (
                    "tax_R16",
                    models.FloatField(
                        default=10.0, verbose_name="Tax for Moon R16 Ore as a percent"
                    ),
                ),
                (
                    "tax_R8",
                    models.FloatField(
                        default=10.0, verbose_name="Tax for Moon R8 Ore as a percent"
                    ),
                ),
                (
                    "tax_R4",
                    models.FloatField(
                        default=10.0, verbose_name="Tax for Moon R4 Ore as a percent"
                    ),
                ),
                (
                    "tax_Gasses",
                    models.FloatField(
                        default=10.0, verbose_name="Tax for Gasses as a percent"
                    ),
                ),
                (
                    "tax_Ice",
                    models.FloatField(
                        default=10.0, verbose_name="Tax for Ice as a percent"
                    ),
                ),
                (
                    "tax_Mercoxit",
                    models.FloatField(
                        default=10.0, verbose_name="Tax for Mercoxit Ore as a percent"
                    ),
                ),
                (
                    "tax_Ores",
                    models.FloatField(
                        default=10.0, verbose_name="Tax for Regular Ore as a percent"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="OrePrices",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("buy", models.FloatField()),
                ("sell", models.FloatField()),
                ("updated", models.DateTimeField()),
                (
                    "eve_type",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="eveuniverse.evetype",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="CharacterUpdateStatus",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "is_success",
                    models.BooleanField(db_index=True, default=None, null=True),
                ),
                ("content_hash_1", models.CharField(default="", max_length=32)),
                ("content_hash_2", models.CharField(default="", max_length=32)),
                ("content_hash_3", models.CharField(default="", max_length=32)),
                ("last_error_message", models.TextField()),
                (
                    "root_task_id",
                    models.CharField(
                        db_index=True,
                        default="",
                        help_text="ID of update_all_characters task that started this update",
                        max_length=36,
                    ),
                ),
                (
                    "parent_task_id",
                    models.CharField(
                        db_index=True,
                        default="",
                        help_text="ID of character_update task that started this update",
                        max_length=36,
                    ),
                ),
                (
                    "started_at",
                    models.DateTimeField(db_index=True, default=None, null=True),
                ),
                (
                    "finished_at",
                    models.DateTimeField(db_index=True, default=None, null=True),
                ),
                (
                    "character",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="update_status_set",
                        to="miningtaxes.character",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="CharacterTaxCredits",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateTimeField(db_index=True)),
                ("credit", models.FloatField(default=0.0)),
                (
                    "character",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="tax_credits",
                        to="miningtaxes.character",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="CharacterMiningLedgerEntry",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateField(db_index=True)),
                ("quantity", models.PositiveIntegerField()),
                ("raw_price", models.FloatField(default=0.0)),
                ("refined_price", models.FloatField(default=0.0)),
                ("taxed_value", models.FloatField(default=0.0)),
                ("taxes_owed", models.FloatField(default=0.0)),
                (
                    "character",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="mining_ledger",
                        to="miningtaxes.character",
                    ),
                ),
                (
                    "eve_solar_system",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveuniverse.evesolarsystem",
                    ),
                ),
                (
                    "eve_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveuniverse.evetype",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AdminMiningObsLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateField(db_index=True)),
                ("miner_id", models.BigIntegerField()),
                ("quantity", models.PositiveIntegerField()),
                ("observer_type", models.CharField(max_length=32)),
                (
                    "eve_solar_system",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveuniverse.evesolarsystem",
                    ),
                ),
                (
                    "eve_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveuniverse.evetype",
                    ),
                ),
                (
                    "observer",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="mining_log",
                        to="miningtaxes.adminminingobservers",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AdminMiningCorpLedgerEntry",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateTimeField(db_index=True)),
                ("taxed_id", models.BigIntegerField()),
                ("amount", models.FloatField(default=0.0)),
                ("reason", models.CharField(max_length=32)),
                (
                    "character",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="corp_ledger",
                        to="miningtaxes.admincharacter",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.AddConstraint(
            model_name="characterupdatestatus",
            constraint=models.UniqueConstraint(
                fields=("character",),
                name="functional_pk_miningtaxes_charactersyncstatus",
            ),
        ),
        migrations.AddConstraint(
            model_name="charactertaxcredits",
            constraint=models.UniqueConstraint(
                fields=("character", "date", "credit"),
                name="functional_pk_miningtaxes_charactertaxes",
            ),
        ),
        migrations.AddConstraint(
            model_name="characterminingledgerentry",
            constraint=models.UniqueConstraint(
                fields=("character", "date", "eve_solar_system", "eve_type"),
                name="functional_pk_mt_characterminingledgerentry",
            ),
        ),
        migrations.AddConstraint(
            model_name="adminminingobslog",
            constraint=models.UniqueConstraint(
                fields=("observer", "date", "miner_id", "eve_type"),
                name="functional_pk_mt_adminMiningLog",
            ),
        ),
        migrations.AddConstraint(
            model_name="adminminingobservers",
            constraint=models.UniqueConstraint(
                fields=("obs_id",), name="functional_pk_mt_adminMiningObs"
            ),
        ),
        migrations.AddConstraint(
            model_name="adminminingcorpledgerentry",
            constraint=models.UniqueConstraint(
                fields=("character", "date", "taxed_id"),
                name="functional_pk_mt_admincorpledger",
            ),
        ),
    ]



from django.core.management.base import BaseCommand

from ...models import AdminCharacter, Character


class Command(BaseCommand):
    help = "Purges characters where their main is not in a tracked corp."

    def handle(self, *args, **options):
        alltoons = AdminCharacter.objects.all()
        validcorps = set()
        for char in alltoons:
            validcorps.add(char.eve_character.corporation_id)

        alltoons = Character.objects.all()

        for char in alltoons:
            try:
                cid = (
                    char.eve_character.character_ownership.user.profile.main_character.corporation_id
                )
            except Exception as e:
                print(f"Exception: {e}")
                char.delete()
                continue

            if cid not in validcorps:
                print(
                    f"Deleting {char.eve_character.character_name} - main: {char.eve_character.character_ownership.user.profile.main_character.character_name}"
                )
                char.delete()



from app_utils.django import clean_setting

MININGTAXES_CORP_WALLET_DIVISION = clean_setting("MININGTAXES_CORP_WALLET_DIVISION", 1)

MININGTAXES_TAX_ONLY_CORP_MOONS = clean_setting("MININGTAXES_TAX_ONLY_CORP_MOONS", True)
"""Only tax corporate moons using moon observers as opposed to all moons appearing
in the personal mining ledgers.
"""

MININGTAXES_UPDATE_LEDGER_STALE = clean_setting("MININGTAXES_UPDATE_LEDGER_STALE", 240)
"""Minutes after which a character's mining ledger is considered stale
"""

MININGTAXES_UPDATE_STALE_OFFSET = clean_setting("MINGINGTAXES_UPDATE_STALE_OFFSET", 5)
"""Actual value for considering staleness of a ring will be the above value
minus this offset. Required to avoid time synchronization issues.
"""

MININGTAXES_TASKS_OBJECT_CACHE_TIMEOUT = clean_setting(
    "MEMBERAUDIT_TASKS_OBJECT_CACHE_TIMEOUT", 600
)

MININGTAXES_TASKS_TIME_LIMIT = clean_setting("MININGTAXES_TASKS_TIME_LIMIT", 7200)
"""Global timeout for tasks in seconds to reduce task accumulation during outages."""

MININGTAXES_REFINED_RATE = clean_setting("MININGTAXES_REFINED_RATE", 0.9063)
"""Refining rate for ores."""

MININGTAXES_ALLOW_ANALYTICS = clean_setting("MININGTAXES_ALLOW_ANALYTICS", True)

MININGTAXES_UNKNOWN_TAX_RATE = clean_setting("MININGTAXES_UNKNOWN_TAX_RATE", 0.10)

MININGTAXES_ALWAYS_TAX_REFINED = clean_setting("MININGTAXES_ALWAYS_TAX_REFINED", False)

MININGTAXES_PRICE_SOURCE_ID = clean_setting("MININGTAXES_PRICE_SOURCE_ID", 60003760)

MININGTAXES_PRICE_SOURCE_NAME = clean_setting("MININGTAXES_PRICE_SOURCE_NAME", "Jita")

MININGTAXES_PRICE_METHOD = clean_setting("MININGTAXES_PRICE_METHOD", "Fuzzwork")

MININGTAXES_PRICE_JANICE_API_KEY = clean_setting("MININGTAXES_PRICE_JANICE_API_KEY", "")

MININGTAXES_BLACKLIST = clean_setting("MININGTAXES_BLACKLIST", [])

MININGTAXES_TAX_HISEC = clean_setting("MININGTAXES_TAX_HISEC", True)
MININGTAXES_TAX_LOSEC = clean_setting("MININGTAXES_TAX_LOSEC", True)
MININGTAXES_TAX_NULLSEC = clean_setting("MININGTAXES_TAX_NULLSEC", True)
MININGTAXES_TAX_JSPACE = clean_setting("MININGTAXES_TAX_JSPACE", True)
MININGTAXES_TAX_POCHVEN = clean_setting("MININGTAXES_TAX_POCHVEN", True)

MININGTAXES_WHITELIST = clean_setting("MININGTAXES_WHITELIST", [])

MININGTAXES_LEADERBOARD_TAXABLE_ONLY = clean_setting(
    "MININGTAXES_LEADERBOARD_TAXABLE_ONLY", False
)

MININGTAXES_PING_FIRST_MSG = clean_setting(
    "MININGTAXES_PING_FIRST_MSG",
    "Please pay {:,.2f} ISK or you will be charged interest!",
)
MININGTAXES_PING_SECOND_MSG = clean_setting(
    "MININGTAXES_PING_SECOND_MSG",
    "Please pay {:,.2f} ISK or you will be charged interest!",
)
MININGTAXES_PING_THRESHOLD = clean_setting("MININGTAXES_PING_THRESHOLD", 0.01)

MININGTAXES_PING_CURRENT_THRESHOLD = clean_setting(
    "MININGTAXES_PING_CURRENT_THRESHOLD", 1000000000
)

MININGTAXES_PING_CURRENT_MSG = clean_setting(
    "MININGTAXES_PING_CURRENT_MSG",
    "Please pay your taxes. Current tax balance: {:,.2f} ISK",
)

MININGTAXES_PING_INTEREST_APPLIED = clean_setting(
    "MININGTAXES_PING_INTEREST_APPLIED",
    "An interest of {:,.2f} ISK has been charged for late taxes.",
)



# Generated by Django 4.0.7 on 2023-05-31 17:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("miningtaxes", "0008_character_monthly_credits_json_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="Stats",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("admin_char_json", models.JSONField(default=None, null=True)),
                ("admin_main_json", models.JSONField(default=None, null=True)),
                ("ore_prices_json", models.JSONField(default=None, null=True)),
                ("admin_mining_by_sys_json", models.JSONField(default=None, null=True)),
                ("admin_tax_revenue_json", models.JSONField(default=None, null=True)),
                ("admin_month_json", models.JSONField(default=None, null=True)),
                ("admin_corp_ledger", models.JSONField(default=None, null=True)),
                (
                    "admin_corp_mining_history",
                    models.JSONField(default=None, null=True),
                ),
                ("leaderboards", models.JSONField(default=None, null=True)),
            ],
        ),
    ]



from django.urls import path

from . import views

app_name = "miningtaxes"

urlpatterns = [
    path("", views.index, name="index"),
    path("launcher", views.launcher, name="launcher"),
    path("add_character", views.add_character, name="add_character"),
    path("add_admin_character", views.add_admin_character, name="add_admin_character"),
    path(
        "remove_character/<int:character_pk>/",
        views.remove_character,
        name="remove_character",
    ),
    path(
        "remove_admin_registered/<int:character_pk>/",
        views.remove_admin_registered,
        name="remove_admin_registered",
    ),
    path(
        "remove_admin_character/<int:character_pk>/",
        views.remove_admin_character,
        name="remove_admin_character",
    ),
    path(
        "character_viewer/<int:character_pk>/",
        views.character_viewer,
        name="character_viewer",
    ),
    path(
        "curmonthgraph",
        views.curmonthgraph,
        name="curmonthgraph",
    ),
    path(
        "purge_old_corphistory",
        views.purge_old_corphistory,
        name="purge_old_corphistory",
    ),
    path(
        "char_mining_ledger_data/<int:character_pk>/",
        views.char_mining_ledger_data,
        name="character_mining_ledger_data",
    ),
    path(
        "user_mining_ledger_90day/<int:user_pk>",
        views.user_mining_ledger_90day,
        name="user_mining_ledger_90day",
    ),
    path("user_summary/<int:user_pk>", views.user_summary, name="user_summary"),
    path(
        "summary_month_json/<int:user_pk>",
        views.summary_month_json,
        name="summary_month_json",
    ),
    path(
        "admin_get_all_activity_json",
        views.admin_get_all_activity_json,
        name="admin_get_all_activity_json",
    ),
    path(
        "all_tax_credits/<int:user_pk>",
        views.all_tax_credits,
        name="all_tax_credits",
    ),
    path("faq", views.faq, name="faq"),
    path("ore_prices", views.ore_prices, name="ore_prices"),
    path("ore_prices_json", views.ore_prices_json, name="ore_prices_json"),
    path("leaderboards", views.leaderboards, name="leaderboards"),
    path("admin/", views.admin_launcher, name="admin_launcher"),
    path(
        "admin_tax_table",
        views.admin_launcher_tax_table,
        name="admin_launcher_tax_table",
    ),
    path(
        "admin_launcher_save_rates",
        views.admin_launcher_save_rates,
        name="admin_launcher_save_rates",
    ),
    path("admin/tables", views.admin_tables, name="admin_tables"),
    path("admin_main_json", views.admin_main_json, name="admin_main_json"),
    path("admin_char_json", views.admin_char_json, name="admin_char_json"),
    path(
        "admin_mining_by_sys_json",
        views.admin_mining_by_sys_json,
        name="admin_mining_by_sys_json",
    ),
    path(
        "admin_tax_revenue_json",
        views.admin_tax_revenue_json,
        name="admin_tax_revenue_json",
    ),
    path("admin_month_json", views.admin_month_json, name="admin_month_json"),
    path(
        "admin_corp_mining_history",
        views.admin_corp_mining_history,
        name="admin_corp_mining_history",
    ),
    path("admin_corp_ledger", views.admin_corp_ledger, name="admin_corp_ledger"),
]



import datetime as dt
from typing import Tuple

from django.contrib.auth.models import User
from django.utils.timezone import now

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter
from allianceauth.tests.auth_utils import AuthUtils
from app_utils.testing import add_character_to_user

from ..models import AdminCharacter, Character, CharacterUpdateStatus


def create_character_update_status(
    character: Character, **kwargs
) -> CharacterUpdateStatus:
    params = {
        "character": character,
        "is_success": True,
        "started_at": now() - dt.timedelta(minutes=5),
        "finished_at": now(),
    }
    params.update(kwargs)
    return CharacterUpdateStatus.objects.create(**params)


def create_character(eve_character, **kwargs) -> Character:
    params = {"eve_character": eve_character}
    params.update(kwargs)
    return Character.objects.create(**params)


def create_character_from_user(user: User, **kwargs):
    """Create new Character object from user. The user needs to have a main character.

    This factory is designed to work with both the old and new variant of Character
    introduced in version 2.
    """
    try:
        character_ownership = user.profile.main_character.character_ownership
    except AttributeError:
        raise ValueError("User needs to have a main character.")
    if hasattr(Character, "eve_character"):
        params = {"eve_character": character_ownership.character}
    else:
        params = {"character_ownership": character_ownership}
    params.update(kwargs)
    return Character.objects.create(**params)


def create_user_from_evecharacter_with_access(
    character_id: int,
) -> Tuple[User, CharacterOwnership]:
    auth_character = EveCharacter.objects.get(character_id=character_id)
    user = AuthUtils.create_user(auth_character.character_name)
    user = AuthUtils.add_permission_to_user_by_name("miningtaxes.basic_access", user)
    character_ownership = add_character_to_user(
        user, auth_character, is_main=True, scopes=Character.get_esi_scopes()
    )
    return user, character_ownership


def create_miningtaxes_character(character_id: int) -> Character:
    _, character_ownership = create_user_from_evecharacter_with_access(character_id)
    return Character.objects.create(eve_character=character_ownership.character)


def create_miningtaxes_admincharacter(character_id: int) -> Character:
    _, character_ownership = create_user_from_evecharacter_with_access(character_id)
    return AdminCharacter.objects.create(eve_character=character_ownership.character)


def add_auth_character_to_user(
    user: User, character_id: int, scopes=None
) -> CharacterOwnership:
    auth_character = EveCharacter.objects.get(character_id=character_id)
    if not scopes:
        scopes = Character.get_esi_scopes()

    return add_character_to_user(user, auth_character, is_main=False, scopes=scopes)


def add_miningtaxes_character_to_user(user: User, character_id: int) -> Character:
    character_ownership = add_auth_character_to_user(user, character_id)
    return Character.objects.create(eve_character=character_ownership.character)



# Generated by Django 4.2.11 on 2024-06-14 00:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("miningtaxes", "0012_stats_user_mining_ledger_90day"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="settings",
            name="tax_Gasses",
        ),
        migrations.RemoveField(
            model_name="settings",
            name="tax_Ice",
        ),
        migrations.RemoveField(
            model_name="settings",
            name="tax_Mercoxit",
        ),
        migrations.RemoveField(
            model_name="settings",
            name="tax_Ores",
        ),
        migrations.RemoveField(
            model_name="settings",
            name="tax_R16",
        ),
        migrations.RemoveField(
            model_name="settings",
            name="tax_R32",
        ),
        migrations.RemoveField(
            model_name="settings",
            name="tax_R4",
        ),
        migrations.RemoveField(
            model_name="settings",
            name="tax_R64",
        ),
        migrations.RemoveField(
            model_name="settings",
            name="tax_R8",
        ),
        migrations.AddField(
            model_name="oreprices",
            name="tax_rate",
            field=models.FloatField(default=10.0),
        ),
    ]
